/*
 * global_status.c
 *
 *  Created on: 04.01.2015
 *      Author: daniel
 */

#include "stm32f4xx.h"
#include "stm32f4xx_it.h"
#include "stm32f4xx_conf.h"
#include "global_status.h"
#include "staticData.h"
#include "core_cm4.h"
#include "io_pins.h"
#include "WS2811.h"
#include "Audio.h"
#include "encoder.h"

typedef struct  {
	uint8_t currentDir;
	uint8_t nextDir;
	bool changeDir;
	bool pause;
	bool skip;
	//bool changeVolume;
	bool playing;
	bool stickDetected;
	int32_t poweroff;
	uint8_t encoderMode;
} GStatus;


GStatus gStatus;

static void setLedStatus();

static void gsEncoderModeVolume() {
	gStatus.encoderMode=0;
	encoderInit(MAXVOLUME, 0, staticDataGetVolume());
}
static void gsEncoderModeToneH() {
	gStatus.encoderMode=1;
	encoderInit(7, -7, staticDataGetToneH());
}
static void gsEncoderModeToneL() {
	gStatus.encoderMode=2;
	encoderInit(7, -7, staticDataGetToneL());
}

void globalStatusInit() {
	ws2811_init();
	setLedStatus();
	gStatus.pause=false;
	gStatus.changeDir=false;
	gStatus.currentDir=255;
	gStatus.nextDir=255;
	gStatus.skip=false;
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.playing=false;
	gStatus.stickDetected=false;
	gsEncoderModeVolume();
	setLedStatus();
}

void globalStatusIdle() {
	if(!gStatus.pause) {
		gStatus.playing=false;
	}
	if(gStatus.poweroff==0) {
		powerOff();
	}
	setLedStatus();
	if(encoderGetChanged()) {
		staticDataSetVolume(encoderGetValue());
	}
}

void globalStatusUpdateVolume() {
	if(encoderGetChanged()) {
		switch (gStatus.encoderMode) {
		case 0:
			staticDataSetVolume(encoderGetValue());
			if(gStatus.playing && (!gStatus.pause)) {
				AudioSetVolume(staticDataGetVolume());
			}
			break;
		case 1:
			staticDataSetToneH(encoderGetValue());
			AudioSetToneH(staticDataGetToneH());
			break;
		case 2:
			staticDataSetToneL(encoderGetValue());
			AudioSetToneL(staticDataGetToneL());
			break;
		}
	}
}


bool globalStatusIsChangeDir() {
	gStatus.poweroff=INITIALPOWEROFF;
	return (gStatus.changeDir);
}

void globalStatusResetChangeDir() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.changeDir=false;
}

void globalStatusSetChangeDir() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.changeDir=true;
}
void globalStatusPlay() {
	gStatus.playing=true;
	gStatus.skip=false;
	gStatus.pause=false;
}
bool globalStatusIsPause() {
	gStatus.poweroff=INITIALPOWEROFF;
	return (gStatus.pause);
}
void globalStatusResetPause() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.pause=false;
}

void globalStatusSetPause() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.pause=true;
}
bool globalStatusIsSkip() {
	gStatus.poweroff=INITIALPOWEROFF;
	return (gStatus.skip);
}
bool globalStatusIsPlaying() {
	return (gStatus.playing);
}
void globalStatusResetSkip() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.skip=false;
}

void globalStatusSetSkip() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.skip=true;
	gStatus.playing=false;
}

void golbalStatusPlayDir(uint8_t dirnumber) {
	gsEncoderModeVolume();
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.playing=true;
	gStatus.skip=false;
	gStatus.pause=false;
	gStatus.nextDir=dirnumber;
	gStatus.changeDir=true;
}

void globalStatusTogglePause() {
	gsEncoderModeVolume();
	if(gStatus.playing) {
		if(gStatus.pause) {
			gStatus.pause=false;
			AudioSetVolume(staticDataGetVolume());
			updateLED();
		} else {
			gStatus.pause=true;
			AudioSetVolume(0);
			updateLED();
		}
	} else {
		gStatus.poweroff=INITIALPOWEROFF;
		gStatus.pause=false;
	}
}

void globalStatusSkip() {
	gsEncoderModeVolume();
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.pause=false;
	gStatus.skip=true;
}

void globalStatusRewind() {
	gsEncoderModeVolume();
	gStatus.poweroff=INITIALPOWEROFF;
	golbalStatusPlayDir(gStatus.currentDir);
	staticDataRewind(gStatus.currentDir);
	gStatus.skip=true;
}

void globalStatusShowActivity() {
	gStatus.poweroff=INITIALPOWEROFF;
	setLedStatus();
	/*
	static uint8_t i=0;
	if(i&0x01) {
		GPIO_SetBits(GPIOD, GPIO_Pin_13);
		GPIO_ResetBits(GPIOD, GPIO_Pin_14);
	} else {
		GPIO_SetBits(GPIOD, GPIO_Pin_14);
		GPIO_ResetBits(GPIOD, GPIO_Pin_13);
	}
	i++;
	 */

}
uint8_t globalStatusGetNextDir() {
	gStatus.poweroff=INITIALPOWEROFF;
	return(gStatus.nextDir);
}
bool globalStatusSelectNextDir() {
	gStatus.poweroff=INITIALPOWEROFF;
	if(gStatus.changeDir) {
		gStatus.currentDir=gStatus.nextDir;
		gStatus.nextDir=255;
		gStatus.changeDir=false;
		return (true);
	} else {
		gStatus.nextDir=255;
		return (false);
	}
}
uint8_t globalStatusGetCurrentDir() {
	gStatus.poweroff=INITIALPOWEROFF;
	return(gStatus.currentDir);
}

void globalStatusSystick() {
	if(gStatus.poweroff>=0) {
		gStatus.poweroff--;
	}
}

void globalStatusPowerOff() {
	gsEncoderModeVolume();
	gStatus.pause=false;
	gStatus.changeDir=false;
	gStatus.currentDir=255;
	gStatus.nextDir=255;
	gStatus.skip=false;
	gStatus.poweroff=INITIALPOWEROFF/100;
	gStatus.playing=false;
}

void globalStatusStickDetected() {
	gStatus.poweroff=INITIALPOWEROFF;
	gStatus.stickDetected=true;
}

static void setLedStatus() {
	static int32_t action=0;
	static bool up=true;
	switch (gStatus.encoderMode) {
	case 0:
		setLED(0,gStatus.poweroff/(INITIALPOWEROFF/10),0,0);
		if(gStatus.pause) {
			setLED_32(1,COLOR_PAUSE);
		} else if (gStatus.playing) {
			if(up) {
				action++;
				if(action>=0x0c*10) {
					up=false;
				}
			} else {
				action--;
				if(action<=0) {
					up=true;
				}
			}
			setLED_32(1,action/10);
		} else {
			if(gStatus.stickDetected) {
				setLED_32(1,COLOR_STANDBY);
			} else {
				setLED_32(1,COLOR_OFF);
			}
		}
		break;
	case 1:
		setLED(0,0,10,0);
		int8_t value=staticDataGetToneH();
		if(value==0) {
			setLED(1,10,0,0);
		}
		if(value>0) {
			setLED(1,0,0,value);
		}
		if(value<0) {
			setLED(1,0,7-value,0);
		}
		break;
	case 2:
		setLED(0,0,0,10);
		value=staticDataGetToneL();
		if(value==0) {
			setLED(1,10,0,0);
		}
		if(value>0) {
			setLED(1,0,0,value);
		}
		if(value<0) {
			setLED(1,0,7-value,0);
		}
		break;
	}

	updateLED();

}

void globalStatusToggleTone() {
	switch (gStatus.encoderMode) {
	case 0:
		gsEncoderModeToneH();
		break;
	case 1:
		gsEncoderModeToneL();
		break;
	case 2:
		gsEncoderModeVolume();
		break;
	}
}
